@@ -56,6 +56,9 @@ typedef struct {
 
 	GArray *wins;
 
+	GArray *nis;
+	char * nis_domain;
+
 	GSList *routes;
 
 	gboolean never_default;
@@ -352,14 +355,21 @@ void nm_ip4_config_reset_routes (NMIP4Config *config)
 
 void nm_ip4_config_add_domain (NMIP4Config *config, const char *domain)
 {
+	NMIP4ConfigPrivate *priv;
+	int i;
+
 	g_return_if_fail (NM_IS_IP4_CONFIG (config));
 	g_return_if_fail (domain != NULL);
 	g_return_if_fail (strlen (domain) > 0);
 
-	if (!strlen (domain))
-		return;
+	priv = NM_IP4_CONFIG_GET_PRIVATE (config);
 
-	g_ptr_array_add (NM_IP4_CONFIG_GET_PRIVATE (config)->domains, g_strdup (domain));
+	for (i = 0; i < priv->domains->len; i++) {
+		if (!strcmp (g_ptr_array_index (priv->domains, i), domain))
+			return;
+	}
+
+	g_ptr_array_add (priv->domains, g_strdup (domain));
 }
 
 const char *nm_ip4_config_get_domain (NMIP4Config *config, guint i)
@@ -376,13 +386,37 @@ guint32 nm_ip4_config_get_num_domains (NMIP4Config *config)
 	return NM_IP4_CONFIG_GET_PRIVATE (config)->domains->len;
 }
 
+void nm_ip4_config_reset_domains (NMIP4Config *config)
+{
+	NMIP4ConfigPrivate *priv;
+	int i;
+
+	g_return_if_fail (NM_IS_IP4_CONFIG (config));
+
+	priv = NM_IP4_CONFIG_GET_PRIVATE (config);
+	for (i = 0; i < priv->domains->len; i++)
+		g_free (g_ptr_array_index (priv->domains, i));
+	g_ptr_array_free (priv->domains, TRUE);
+	priv->domains = g_ptr_array_sized_new (3);
+}
+
 void nm_ip4_config_add_search (NMIP4Config *config, const char *search)
 {
+	NMIP4ConfigPrivate *priv;
+	int i;
+
 	g_return_if_fail (config != NULL);
 	g_return_if_fail (search != NULL);
 	g_return_if_fail (strlen (search) > 0);
 
-	g_ptr_array_add (NM_IP4_CONFIG_GET_PRIVATE (config)->searches, g_strdup (search));
+	priv = NM_IP4_CONFIG_GET_PRIVATE (config);
+
+	for (i = 0; i < priv->searches->len; i++) {
+		if (!strcmp (g_ptr_array_index (priv->searches, i), search))
+			return;
+	}
+
+	g_ptr_array_add (priv->searches, g_strdup (search));
 }
 
 const char *nm_ip4_config_get_search (NMIP4Config *config, guint i)
@@ -402,12 +436,15 @@ guint32 nm_ip4_config_get_num_searches (NMIP4Config *config)
 void nm_ip4_config_reset_searches (NMIP4Config *config)
 {
 	NMIP4ConfigPrivate *priv;
+	int i;
 
 	g_return_if_fail (NM_IS_IP4_CONFIG (config));
 
 	priv = NM_IP4_CONFIG_GET_PRIVATE (config);
-	if (priv->searches->len)
-		g_ptr_array_remove_range (priv->searches, 0, priv->searches->len);
+	for (i = 0; i < priv->searches->len; i++)
+		g_free (g_ptr_array_index (priv->searches, i));
+	g_ptr_array_free (priv->searches, TRUE);
+	priv->searches = g_ptr_array_sized_new (3);
 }
 
 guint32 nm_ip4_config_get_mtu (NMIP4Config *config)
@@ -454,6 +491,70 @@ nm_ip4_config_set_never_default (NMIP4Config *config, gboolean never_default)
 	NM_IP4_CONFIG_GET_PRIVATE (config)->never_default = never_default;
 }
 
+void nm_ip4_config_add_nis_server (NMIP4Config *config, guint32 nis)
+{
+	NMIP4ConfigPrivate *priv;
+	int i;
+
+	g_return_if_fail (NM_IS_IP4_CONFIG (config));
+	g_return_if_fail (nis > 0);
+
+	priv = NM_IP4_CONFIG_GET_PRIVATE (config);
+	for (i = 0; i < priv->nis->len; i++) {
+		guint32 s = g_array_index (priv->nis, guint32, i);
+
+		/* No dupes */
+		g_return_if_fail (nis != s);
+	}
+
+	g_array_append_val (priv->nis, nis);
+}
+
+guint32 nm_ip4_config_get_nis_server (NMIP4Config *config, guint i)
+{
+	g_return_val_if_fail (NM_IS_IP4_CONFIG (config), 0);
+
+	return g_array_index (NM_IP4_CONFIG_GET_PRIVATE (config)->nis, guint32, i);
+}
+
+guint32 nm_ip4_config_get_num_nis_servers (NMIP4Config *config)
+{
+	g_return_val_if_fail (NM_IS_IP4_CONFIG (config), 0);
+
+	return NM_IP4_CONFIG_GET_PRIVATE (config)->nis->len;
+}
+
+void nm_ip4_config_reset_nis_servers (NMIP4Config *config)
+{
+	NMIP4ConfigPrivate *priv;
+
+	g_return_if_fail (NM_IS_IP4_CONFIG (config));
+
+	priv = NM_IP4_CONFIG_GET_PRIVATE (config);
+	if (priv->nis->len)
+		g_array_remove_range (priv->nis, 0, priv->nis->len);
+}
+
+void
+nm_ip4_config_set_nis_domain (NMIP4Config *config, const char *domain)
+{
+	NMIP4ConfigPrivate *priv;
+
+	g_return_if_fail (NM_IS_IP4_CONFIG (config));
+
+	priv = NM_IP4_CONFIG_GET_PRIVATE (config);
+	g_free (priv->nis_domain);
+	priv->nis_domain = g_strdup (domain);
+}
+
+const char *
+nm_ip4_config_get_nis_domain (NMIP4Config *config)
+{
+	g_return_val_if_fail (NM_IS_IP4_CONFIG (config), 0);
+
+	return NM_IP4_CONFIG_GET_PRIVATE (config)->nis_domain;
+}
+
 /* libnl convenience/conversion functions */
 
 static int ip4_addr_to_rtnl_local (guint32 ip4_address, struct rtnl_addr *addr)
@@ -666,6 +767,15 @@ nm_ip4_config_diff (NMIP4Config *a, NMIP4Config *b)
 	    || !addr_array_compare (b_priv->wins, a_priv->wins))
 		flags |= NM_IP4_COMPARE_FLAG_WINS_SERVERS;
 
+	if (   (a_priv->nis->len != b_priv->nis->len)
+	    || !addr_array_compare (a_priv->nis, b_priv->nis)
+	    || !addr_array_compare (b_priv->nis, a_priv->nis))
+		flags |= NM_IP4_COMPARE_FLAG_NIS_SERVERS;
+
+	if (   (a_priv->nis_domain || b_priv->nis_domain)
+		&& (g_strcmp0 (a_priv->nis_domain, b_priv->nis_domain) != 0))
+		flags |= NM_IP4_COMPARE_FLAG_NIS_DOMAIN;
+
 	if (   !route_slist_compare (a_priv->routes, b_priv->routes)
 	    || !route_slist_compare (b_priv->routes, a_priv->routes))
 		flags |= NM_IP4_COMPARE_FLAG_ROUTES;
@@ -696,8 +806,9 @@ nm_ip4_config_init (NMIP4Config *config)
 
 	priv->nameservers = g_array_new (FALSE, TRUE, sizeof (guint32));
 	priv->wins = g_array_new (FALSE, TRUE, sizeof (guint32));
-	priv->domains = g_ptr_array_new ();
-	priv->searches = g_ptr_array_new ();
+	priv->domains = g_ptr_array_sized_new (3);
+	priv->searches = g_ptr_array_sized_new (3);
+	priv->nis = g_array_new (FALSE, TRUE, sizeof (guint32));
 }
 
 static void
@@ -711,6 +822,8 @@ finalize (GObject *object)
 	g_array_free (priv->nameservers, TRUE);
 	g_ptr_array_free (priv->domains, TRUE);
 	g_ptr_array_free (priv->searches, TRUE);
+	g_array_free (priv->nis, TRUE);
+	g_free (priv->nis_domain);
 
 	G_OBJECT_CLASS (nm_ip4_config_parent_class)->finalize (object);
 }
